@@ -2,14 +2,13 @@
 # -*- coding: utf-8 -*-
 # Copyright (c) 2011-2012 Raphaël Barrois
 
-from lcdproc import server as lcdproc_server
-
 import ConfigParser
 import logging
 from logging import handlers as logging_handlers
 import optparse
 import socket
 import time
+import sys
 
 from mpdlcd import lcdrunner
 from mpdlcd import mpdwrapper
@@ -28,6 +27,7 @@ DEFAULT_PATTERN = ''
 # Connection
 DEFAULT_MPD_PORT = 6600
 DEFAULT_LCD_PORT = 13666
+DEFAULT_LCDPROC_CHARSET = 'iso-8859-1'
 DEFAULT_RETRY_ATTEMPTS = 3
 DEFAULT_RETRY_WAIT = 3
 DEFAULT_RETRY_BACKOFF = 2
@@ -49,6 +49,7 @@ BASE_CONFIG = {
     'connections': {
         'mpd': ('str', 'localhost:%s' % DEFAULT_MPD_PORT),
         'lcdproc': ('str', 'localhost:%s' % DEFAULT_LCD_PORT),
+        'lcdproc_charset': ('str', DEFAULT_LCDPROC_CHARSET),
         'lcdd_debug': ('bool', False),
         'retry_attempts': ('int', DEFAULT_RETRY_ATTEMPTS),
         'retry_wait': ('int', DEFAULT_RETRY_WAIT),
@@ -59,29 +60,29 @@ BASE_CONFIG = {
         'loglevel': ('str', DEFAULT_LOGLEVEL),
         'syslog_facility': ('str', DEFAULT_SYSLOG_FACILITY),
         'syslog_address': ('str', DEFAULT_SYSLOG_ADDRESS),
-        'logfile': ('str', DEFAULT_CONFIG_FILE),
+        'logfile': ('str', DEFAULT_LOGFILE),
         'debug': ('str', DEFAULT_DEBUG_MODULES),
     },
 }
 
 DEFAULT_PATTERNS = [
     # One line
-    """{state} {song format="%(artist)s - %(title)s"} {elapsed}""",
+    u"""{state} {song format="%(artist)s - %(title)s"} {elapsed}""",
 
     # Two lines
-    """{song format="%(artist)s",speed=4} {elapsed}\n"""
-    """{song format="%(title)s",speed=2} {state}""",
+    u"""{song format="%(artist)s",speed=4} {elapsed}\n"""
+    u"""{song format="%(title)s",speed=2} {state}""",
 
     # Three lines
-    """{song format="%(artist)s",speed=4}\n"""
-    """{song format="%(album)s - %(title)s",speed=2}\n"""
-    """{state}  {elapsed} / {total}""",
+    u"""{song format="%(artist)s",speed=4}\n"""
+    u"""{song format="%(album)s - %(title)s",speed=2}\n"""
+    u"""{state}  {elapsed} / {total}""",
 
     # Four lines
-    """{song format="%(artist)s",speed=4}\n"""
-    """{song format="%(album)s",speed=4}\n"""
-    """{song format="%(title)s",speed=2}\n"""
-    """{elapsed}  {state}  {remaining}""",
+    u"""{song format="%(artist)s",speed=4}\n"""
+    u"""{song format="%(album)s",speed=4}\n"""
+    u"""{song format="%(title)s",speed=2}\n"""
+    u"""{elapsed}  {state}  {remaining}""",
 ]
 
 
@@ -116,12 +117,14 @@ def _make_hostport(conn, default_host, default_port):
     return host, int(port)
 
 
-def _make_lcdproc(lcd_host, lcd_port, retry_config, lcdd_debug=False):
+def _make_lcdproc(lcd_host, lcd_port, retry_config,
+        charset=DEFAULT_LCDPROC_CHARSET, lcdd_debug=False):
     """Create and connect to the LCDd server.
 
     Args:
         lcd_host (str): the hostname to connect to
         lcd_prot (int): the port to connect to
+        charset (str): the charset to use when sending messages to lcdproc
         lcdd_debug (bool): whether to enable full LCDd debug
         retry_attempts (int): the number of connection attempts
         retry_wait (int): the time to wait between connection attempts
@@ -136,14 +139,15 @@ def _make_lcdproc(lcd_host, lcd_port, retry_config, lcdd_debug=False):
 
         @utils.auto_retry
         def connect(self):
-            return lcdproc_server.Server(lcd_host, lcd_port, debug=lcdd_debug)
+            return lcdrunner.LcdProcServer(
+                lcd_host, lcd_port, charset, debug=lcdd_debug)
 
     spawner = ServerSpawner(retry_config=retry_config, logger=logger)
 
     try:
         return spawner.connect()
     except socket.error as e:
-        logger.error('Unable to connect to lcdproc %s:%s.',
+        logger.error(u'Unable to connect to lcdproc %s:%s.',
             lcd_host, lcd_port)
         raise SystemExit(1)
 
@@ -166,6 +170,7 @@ def _make_patterns(patterns):
 
 
 def run_forever(lcdproc='', mpd='', lcdproc_screen=DEFAULT_LCD_SCREEN_NAME,
+        lcdproc_charset=DEFAULT_LCDPROC_CHARSET,
         lcdd_debug=False,
         pattern='', patterns=[],
         retry_attempts=DEFAULT_RETRY_ATTEMPTS,
@@ -177,6 +182,7 @@ def run_forever(lcdproc='', mpd='', lcdproc_screen=DEFAULT_LCD_SCREEN_NAME,
         lcdproc (str): the target connection (host:port) for lcdproc
         mpd (str): the target connection (host:port) for mpd
         lcdproc_screen (str): the name of the screen to use for lcdproc
+        lcdproc_charset (str): the charset to use with lcdproc
         lcdd_debug (bool): whether to enable full LCDd debug
         pattern (str): the pattern to use
         patterns (str list): the patterns to use
@@ -200,7 +206,7 @@ def run_forever(lcdproc='', mpd='', lcdproc_screen=DEFAULT_LCD_SCREEN_NAME,
 
     # Setup LCDd client
     lcd = _make_lcdproc(lcd_host, lcd_port, lcdd_debug=lcdd_debug,
-        retry_config=retry_config)
+        charset=lcdproc_charset, retry_config=retry_config)
 
     # Setup connector
     runner = lcdrunner.MpdRunner(mpd_client, lcd, lcdproc_screen=lcdproc_screen,
@@ -273,6 +279,8 @@ def _make_parser():
             help='Connect to lcdproc at LCDPROC', metavar='LCDPROC')
     group.add_option('-m', '--mpd', dest='mpd',
             help='Connect to mpd running at MPD', metavar='MPD')
+    group.add_option('--lcdproc-charset', dest='lcdproc_charset',
+            help='Use CHARSET for communications to lcdproc', metavar='CHARSET')
     group.add_option('--lcdd-debug', dest='lcdd_debug', action='store_true',
             help='Add full debug output of LCDd commands', default=False)
 
@@ -298,6 +306,8 @@ def _make_parser():
     group = optparse.OptionGroup(parser, 'Logging')
     group.add_option('-s', '--syslog', dest='syslog', action='store_true',
             help='Enable syslog logging (default: False)')
+    group.add_option('--no-syslog', dest='syslog', action='store_false',
+            help='Disable syslog logging (Useful when enabled in config file)')
 
     group.add_option('--syslog-facility', dest='syslog_facility',
             help='Log into syslog facility FACILITY (default: %s)' %
@@ -349,7 +359,7 @@ def _setup_logging(syslog=False, syslog_facility=DEFAULT_SYSLOG_FACILITY,
         handler.setFormatter(quiet_formatter)
 
     else:
-        handler = logging.FileHandler(logfile, level=level)
+        handler = logging.FileHandler(logfile)
         handler.setFormatter(verbose_formatter)
 
     root_logger = logging.getLogger()
@@ -361,18 +371,30 @@ def _setup_logging(syslog=False, syslog_facility=DEFAULT_SYSLOG_FACILITY,
             continue
         module_logger = logging.getLogger(module)
         module_logger.setLevel(logging.DEBUG)
-        module_logger.addHandler(handler)
+        if level != logging.DEBUG:
+            # Make sure log messages are displayed
+            module_logger.addHandler(handler)
         module_logger.info("Enabling debug")
 
 
 def _read_config(filename):
+    """Read configuration from the given file.
+
+    Parsing is performed through the ConfigParser library.
+
+    Returns:
+        dict: a flattened dict of (option_name, value), using defaults.
+    """
     parser = ConfigParser.RawConfigParser()
-    if not parser.read(filename):
-        return
+    if filename and not parser.read(filename):
+        sys.stderr.write(
+            u"Unable to open configuration file %s. Use --config='' to disable "
+            u"this warning.\n" % filename)
 
     config = {}
 
     for section, defaults in BASE_CONFIG.iteritems():
+        # Patterns are handled separately
         if section == 'patterns':
             continue
 
@@ -401,10 +423,24 @@ def _read_config(filename):
 
 
 def _extract_options(config, options, *args):
+    """Extract options values from a ConfigParser, optparse pair.
+
+    Options given on command line take precedence over options read in the
+    configuration file.
+
+    Args:
+        config (dict): option values read from a config file through
+            ConfigParser
+        options (optparse.Options): optparse 'options' object containing options
+            values from the command line
+        *args (str tuple): name of the options to extract
+    """
     extract = {}
     for key in args:
+        if key not in args:
+            continue
         extract[key] = config[key]
-        option = getattr(options, key)
+        option = getattr(options, key, None)
         if option is not None:
             extract[key] = option
     return extract
@@ -422,5 +458,5 @@ def main(argv):
         'syslog', 'syslog_facility', 'syslog_address',
         'logfile', 'loglevel', 'debug'))
     run_forever(**_extract_options(base_config, options,
-        'lcdproc', 'mpd', 'lcdproc_screen', 'lcdd_debug', 'pattern', 'patterns',
-        'retry_attempts', 'retry_backoff', 'retry_wait'))
+        'lcdproc', 'mpd', 'lcdproc_charset', 'lcdproc_screen', 'lcdd_debug',
+        'pattern', 'patterns', 'retry_attempts', 'retry_backoff', 'retry_wait'))
