@@ -33,10 +33,19 @@ class MPDClient(utils.AutoRetryCandidate):
         self.host = host
         self.port = port
 
+    def _decode_text(self, text):
+        # MPD protocol states that all data is UTF-8 encoded.
+        # Ref: http://www.musicpd.org/doc/protocol/ch01s02.html
+        return unicode(text, 'utf8')
+
+    def _decode_dict(self, data):
+        return dict(
+            (k, self._decode_text(v)) for k, v in data.items())
+
     @utils.auto_retry
     def connect(self):
         if not self._connected:
-            logger.info('Connecting to MPD server at %s:%s', self.host, self.port)
+            logger.info(u'Connecting to MPD server at %s:%s', self.host, self.port)
             self._client.connect(host=self.host, port=self.port)
             self._connected = True
 
@@ -47,12 +56,12 @@ class MPDClient(utils.AutoRetryCandidate):
 
     @property
     def random(self):
-        logger.debug('Fetching MPD random state')
+        logger.debug(u'Fetching MPD random state')
         return self.status['random'] == 1
 
     @property
     def repeat(self):
-        logger.debug('Fetchin MPD repeat state')
+        logger.debug(u'Fetchin MPD repeat state')
         return self.status['repeat'] == 1
 
     def _parse_time(self, time):
@@ -63,7 +72,7 @@ class MPDClient(utils.AutoRetryCandidate):
 
     @property
     def elapsed(self):
-        logger.debug('Fetching MPD elapsed time')
+        logger.debug(u'Fetching MPD elapsed time')
         time = self.status.get('time')
         if time:
             return self._parse_time(time.split(':')[0])
@@ -72,7 +81,7 @@ class MPDClient(utils.AutoRetryCandidate):
 
     @property
     def total(self):
-        logger.debug('Fetching MPD total time')
+        logger.debug(u'Fetching MPD total time')
         time = self.status.get('time')
         if time:
             return self._parse_time(time.split(':')[-1])
@@ -81,7 +90,7 @@ class MPDClient(utils.AutoRetryCandidate):
 
     @property
     def elapsed_and_total(self):
-        logger.debug('Fetching MPD elapsed and total time')
+        logger.debug(u'Fetching MPD elapsed and total time')
         time = self.status.get('time')
         if time and ':' in time:
             elapsed, total = time.split(':', 1)
@@ -91,24 +100,37 @@ class MPDClient(utils.AutoRetryCandidate):
 
     @property
     def state(self):
-        logger.debug('Fetching MPD state')
+        logger.debug(u'Fetching MPD state')
         return self.status['state']
 
     @property
     @utils.auto_retry
     def current_song(self):
-        logger.debug('Fetching MPD song information')
-        return MPDSong(**self._client.currentsong())
+        logger.debug(u'Fetching MPD song information')
+        return MPDSong(**self._decode_dict(self._client.currentsong()))
 
 
 class MPDSong(object):
+    DEFAULTS = {
+        'artist': u"<Unknown>",
+        'title': u"<Unknown>",
+        'album': u"<Unknown>",
+        'track': u"0",
+        'file': u"<Unknown>",
+    }
 
     def __init__(self, **kwargs):
-        for k, v in kwargs.iteritems():
+        defaults = dict(self.DEFAULTS)
+        defaults.update(kwargs)
+        for k, v in defaults.iteritems():
             setattr(self, k, v)
 
-    def format(self, fmt='{artist} - {title}'):
+    def __nonzero__(self):
+        """If no song is playing, we won't have an ID."""
+        return hasattr(self, 'id')
+
+    def format(self, fmt=u'{artist} - {title}'):
         return fmt.format(
-                title=self.title,
-                artist=self.artist,
-                )
+            title=self.title,
+            artist=self.artist,
+        )
